@@ -1,10 +1,13 @@
 import re
 from flask import Blueprint, request
+from flask.globals import _app_ctx_stack, _request_ctx_stack
+from werkzeug.urls import url_parse
 from flask.ext.babelpkg import gettext, ngettext, lazy_gettext
 from forms import GeneralModelConverter
 from .widgets import FormWidget, ShowWidget, ListWidget, SearchWidget, ListCarousel
 from .models.filters import Filters, FilterRelationOneToManyEqual
 from .actions import ActionItem
+from urltools import *
 
 def expose(url='/', methods=('GET',)):
     """
@@ -134,12 +137,38 @@ class BaseView(object):
         return name.replace('_', ' ').title()
 
 
+    def route_from(url, method = None):
+        appctx = _app_ctx_stack.top
+        reqctx = _request_ctx_stack.top
+        if appctx is None:
+            raise RuntimeError('Attempted to match a URL without the '
+                           'application context being pushed. This has to be '
+                           'executed when application context is available.')
+
+        if reqctx is not None:
+            url_adapter = reqctx.url_adapter
+        else:
+            url_adapter = appctx.url_adapter
+            if url_adapter is None:
+                raise RuntimeError('Application was not able to create a URL '
+                               'adapter for request independent URL matching. '
+                               'You might be able to fix this by setting '
+                               'the SERVER_NAME config variable.')
+        parsed_url = url_parse(url)
+        if parsed_url.netloc is not "" and parsed_url.netloc != url_adapter.server_name:
+            raise NotFound()
+        return url_adapter.match(parsed_url.path, method)
+
     def _get_redirect(self):
-        if (request.args.get('next')):
-            return request.args.get('next')
+        next_url = request.args.get('next')
+        if (next_url):
+            if next_url in request.referrer:
+                return request.referrer
+            else:
+                return request.args.get('next')
         else:
             try:
-                return url_for('%s.%s' % (self.endpoint, self.default_view))
+                return url_for('%s.%s' % (self.endpoint, self.default_view), **request.args)
             except:
                 return url_for('%s.%s' % (self.baseapp.indexview.endpoint, self.baseapp.indexview.default_view))
             
@@ -426,6 +455,11 @@ class BaseCRUDView(BaseModelView):
                 self.edit_columns = list_cols
         
 
+    """
+    -----------------------------------------------------
+            GET WIDGETS SECTION
+    -----------------------------------------------------        
+    """
     def _get_related_list_widget(self, item, related_view, 
                                 order_column='', order_direction='',
                                 page=None, page_size=None):
@@ -438,7 +472,10 @@ class BaseCRUDView(BaseModelView):
                     order_direction = order_direction,
                     page=page, page_size=page_size)
 
-    def _get_related_list_widgets(self, item, orders = {}, pages=None, page_sizes=None, widgets = {}, **args):
+    def _get_related_list_widgets(self, item, orders = None, 
+                                pages=None, page_sizes=None, 
+                                widgets = None, **args):
+        widgets = widgets or {}
         widgets['related_lists'] = []
         for view in self.related_views:
             if orders.get(view.__class__.__name__):
@@ -455,9 +492,10 @@ class BaseCRUDView(BaseModelView):
                         order_direction = '',
                         page = None,
                         page_size = None,
-                        widgets = {}, **args):
+                        widgets = None, **args):
 
         """ get joined base filter and current active filter for query """
+        widgets = widgets or {}
         actions = actions or self.actions
         page_size = page_size or self.page_size
         if not order_column and self.base_order:
@@ -465,19 +503,18 @@ class BaseCRUDView(BaseModelView):
         joined_filters = filters.get_joined_filters(self._base_filters)
         count, lst = self.datamodel.query(joined_filters, order_column, order_direction, page=page, page_size=page_size)
         pks = self.datamodel.get_keys(lst)
-        widgets['list'] = self.list_widget(route_base = self.route_base,
-                                                label_columns = self.label_columns,
-                                                include_columns = self.list_columns,
-                                                value_columns = self.datamodel.get_values(lst, self.list_columns),
-                                                order_columns = self.order_columns,
-                                                page = page,
-                                                page_size = page_size,
-                                                count = count,
-                                                pks = pks,
-                                                actions = actions,
-                                                filters = filters,
-                                                generalview_name = self.__class__.__name__
-                                                )
+        widgets['list'] = self.list_widget(label_columns = self.label_columns,
+                                            include_columns = self.list_columns,
+                                            value_columns = self.datamodel.get_values(lst, self.list_columns),
+                                            order_columns = self.order_columns,
+                                            page = page,
+                                            page_size = page_size,
+                                            count = count,
+                                            pks = pks,
+                                            actions = actions,
+                                            filters = filters,
+                                            generalview_name = self.__class__.__name__
+                                            )
         return widgets
 
 
@@ -485,41 +522,156 @@ class BaseCRUDView(BaseModelView):
         widgets = widgets or {}
         actions = actions or self.actions
         item = self.datamodel.get(id)
-        widgets['show'] = self.show_widget(route_base = self.route_base,
-                                                pk = id,
-                                                label_columns = self.label_columns,
-                                                include_columns = self.show_columns,
-                                                value_columns = self.datamodel.get_values_item(item, self.show_columns),
-                                                actions = actions,
-                                                fieldsets = self.show_fieldsets,
-                                                generalview_name = self.__class__.__name__
-                                                )
+        widgets['show'] = self.show_widget(pk = id,
+                                            label_columns = self.label_columns,
+                                            include_columns = self.show_columns,
+                                            value_columns = self.datamodel.get_values_item(item, self.show_columns),
+                                            actions = actions,
+                                            fieldsets = self.show_fieldsets,
+                                            generalview_name = self.__class__.__name__
+                                            )
         return widgets
 
 
-    def _get_add_widget(self, form = None, exclude_cols = None, widgets = None):
+    def _get_add_widget(self, form, exclude_cols = None, widgets = None):
         exclude_cols = exclude_cols or []
         widgets = widgets or {}
-        widgets['add'] = self.edit_widget(route_base = self.route_base,
-                                                form = form,
-                                                include_cols = self.add_columns,
-                                                exclude_cols = exclude_cols,
-                                                fieldsets = self.add_fieldsets
-                                                )
+        widgets['add'] = self.add_widget(form = form,
+                                            include_cols = self.add_columns,
+                                            exclude_cols = exclude_cols,
+                                            fieldsets = self.add_fieldsets
+                                            )
         return widgets
 
-    def _get_edit_widget(self, form = None, exclude_cols = None, widgets = None):
+    def _get_edit_widget(self, form, exclude_cols = None, widgets = None):
         exclude_cols = exclude_cols or []
         widgets = widgets or {}
-        widgets['edit'] = self.edit_widget(route_base = self.route_base,
-                                                form = form,
-                                                include_cols = self.edit_columns,
-                                                exclude_cols = exclude_cols,
-                                                fieldsets = self.edit_fieldsets
-                                                )
+        widgets['edit'] = self.edit_widget(form = form,
+                                            include_cols = self.edit_columns,
+                                            exclude_cols = exclude_cols,
+                                            fieldsets = self.edit_fieldsets
+                                            )
         return widgets
 
 
+    """
+    -----------------------------------------------------
+            CRUD functions behaviour
+    -----------------------------------------------------        
+    """
+    def _list(self):
+        """
+            list function logic, override to implement diferent logic
+            returns list and search widget
+        """
+        if get_order_args().get(self.__class__.__name__):
+            order_column, order_direction = get_order_args().get(self.__class__.__name__)
+        else: order_column, order_direction = '',''
+        page = get_page_args().get(self.__class__.__name__)
+        page_size = get_page_size_args().get(self.__class__.__name__)
+        get_filter_args(self._filters)
+        
+        widgets = self._get_list_widget(filters = self._filters, 
+                    order_column = order_column, 
+                    order_direction = order_direction, 
+                    page = page, 
+                    page_size = page_size)
+        form = self.search_form.refresh()
+        return self._get_search_widget(form = form, widgets = widgets)
+
+
+    def _show(self, pk):
+        """
+            show function logic, override to implement diferent logic
+            returns show and related list widget
+        """
+        pages = get_page_args()
+        page_sizes = get_page_size_args()
+        orders = get_order_args()
+
+        widgets = self._get_show_widget(pk)
+        item = self.datamodel.get(pk)
+                
+        return self._get_related_list_widgets(item, orders = orders, 
+                pages = pages, page_sizes = page_sizes, widgets = widgets)
+
+
+    def _add(self):
+        """
+            Add function logic, override to implement diferent logic
+            returns add widget or None
+        """
+        get_filter_args(self._filters)
+        form = self.add_form.refresh()
+        exclude_cols = self._filters.get_relation_cols()
+
+        if form.validate_on_submit():
+            item = self.datamodel.obj()
+            form.populate_obj(item)
+            for filter_key in exclude_cols:
+                rel_obj = self.datamodel.get_related_obj(filter_key, self._filters.get_filter_value(filter_key))
+                setattr(item, filter_key, rel_obj)
+
+            self.pre_add(item)
+            self.datamodel.add(item)
+            self.post_add(item)
+            return None
+        else:
+            return self._get_add_widget(form = form, exclude_cols = exclude_cols)
+
+    def _edit(self, pk):
+        """
+            Edit function logic, override to implement diferent logic
+            returns Edit widget and related list or None
+        """
+
+        pages = get_page_args()
+        page_sizes = get_page_size_args()
+        orders = get_order_args()
+        
+        item = self.datamodel.get(pk)
+        # convert pk to correct type, if pk is non string type.
+        pk = self.datamodel.get_pk_value(item)
+        get_filter_args(self._filters)
+        exclude_cols = self._filters.get_relation_cols()
+
+        if request.method == 'POST':
+            form = self.edit_form(request.form)
+            form = form.refresh(obj=item)
+            # trick to pass unique validation
+            form._id = pk
+            if form.validate():
+                form.populate_obj(item)
+                # fill the form with the suppressed cols, generated from exclude_cols
+                for filter_key in exclude_cols:
+                    rel_obj = self.datamodel.get_related_obj(filter_key, self._filters.get_filter_value(filter_key))
+                    setattr(item, filter_key, rel_obj)
+                
+                self.pre_update(item)
+                self.datamodel.edit(item)
+                self.post_update(item)
+                return None
+            else:
+                widgets = self._get_edit_widget(form = form, exclude_cols = exclude_cols)
+                widgets = self._get_related_list_widgets(item, filters = {}, 
+                        orders = orders, pages = pages, page_sizes=page_sizes, widgets = widgets)
+                return widgets
+        else:
+            form = self.edit_form(obj=item)
+            form = form.refresh(obj=item)
+            widgets = self._get_edit_widget(form = form, exclude_cols = exclude_cols)
+            widgets = self._get_related_list_widgets(item, filters = {}, 
+                        orders = orders, pages = pages, page_sizes=page_sizes, widgets = widgets)                
+            return widgets
+
+
+    def _delete(self, pk):
+        item = self.datamodel.get(pk)
+        self.pre_delete(item)
+        self.datamodel.delete(item)
+        self.post_delete(item)        
+        
+
     def debug(self):
 
         print self.__class__.__name__, "SHOW FS", self.show_fieldsets
